/*
 *  MULTI2 Descrambler for BCAS
 *
 *  Copyright (C) 2009 0p1pp1 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.=
 */
#ifndef _BCAS_CARDIO_H_
#define _BCAS_CARDIO_H_

#include <stdint.h>
#include <pthread.h>
#include "isdb_demulti2.h"

#define BCAS_MAX_CARDID 8
#define BCAS_MAX_MSG_LEN 266

typedef struct _bcas_card {
    char *iccname;
	uint16_t cas_id;
	enum {CARD_S_NG, CARD_S_OK} status;
	int num_id;
	uint8_t id[BCAS_MAX_CARDID][10]; 
	struct demulti2_param param;
	/* PC/SC card handles etc. used by bcas_card_*() */
	void *io_arg;

	/* command buffer to the BCAS PC card */
	uint8_t cmdbuf[BCAS_MAX_MSG_LEN];
	/* length of the command  & the command processing status */
	/* if cmd_len > 0: command is issued, and waiting for the resp. */
	/* if cmd_len == 0: ready to accept new one. */
	volatile int cmd_len;
	/* callback func called when the response from the card is received */
	void  (*cmd_cb)(struct _bcas_card *bcas, void *kinfo,
		uint8_t *rbuf, size_t rlen);
	void *cmd_arg;

	pthread_t thread;
	pthread_mutex_t lock;
	pthread_cond_t cond;
} bcas_card_t;

#define BCAS_THREADED_IO 1
#ifndef BCAS_THREADED_IO
extern void *bcas_card_io(void *data);
#endif
extern bcas_card_t *bcas_card_init(void);
extern void bcas_card_stop(bcas_card_t * card);
extern void  bcas_ecm_cb(struct _bcas_card *bcas, void *kinfo,
		uint8_t *rbuf, size_t rlen);

#endif // _BCAS_CARDIO_H_
